/*
BSL-Scripter-3.1.0.0
			  
==BSL_Scripter.cpp==
					
Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa
3.1.0.0		Oct/26/2015		Fatmawati Santosa
*/

#include "stdafx.h"
#include "hidapi.h"
#include <stdio.h>


/*******************************************************************************
*Global Variables
*******************************************************************************/
ScripterStream   scripterStream;  // Scripter stream class
Interpreter      interpreter;     // Interpreter class 
FactoryTarget    factoryTarget;   // FactoryTarget class
TargetInterface *targetInterface; // Pointer to TargetInterface
FactoryComm      factoryComm;     // FactoryComm class
CommInterface   *commInterface;   // Pointer to CommInterface

std::string mainPathDir;          // Path of executed BSL Scripter
uint32_t    numOfLines;		      // Number of command lines
uint8_t*    ptrRxBuffer;          // Pointer to the receive buffer
std::string getResponseString;	  // Translated BSL device response
 
Timer       timer;                // Timer class
double      totalTime;			  // Total elapsed time for transmission process
   
/*******************************************************************************
*Function declaration
*******************************************************************************/
void execModeParameters();
void getRepeatModeParameters(Family prevFamSetting);
void executeBslCommand(uint16_t numBytesCommand);
void initReadingTxtFile();
uint16_t initWritingTxtFile(uint32_t numBytesWrite);
void showVerbose(bool enVerbose);
void handleResponse();
void changeBaudRate();
int32_t exitProgramError(std::string errorMessage);
int32_t exitProgram();

/*******************************************************************************
*Main Program
*******************************************************************************/
int main(int argc, char * argv[])
{
	// Start the scripter 
	// If the input file is not inserted as an argument, it will ask the script file name
	try
	{
		mainPathDir = scripterStream.startScripter(argv[1]);
	}
	catch (std::runtime_error& e)
	{
		scripterStream.writeLogFile(e.what(),  false);
		return (EXIT_PROGRAM);
	}

	// Initialize number of lines that been read by the scripterStream
	// It is required to check that the MODE command should be called first
	numOfLines = 0;
	// Initialize that the Log file is not created yet
	bool logFileCreated = false;
	
	// Check if line is available
	while (scripterStream.getLine())
	{
		// Clear the input parameters from previous command
		interpreter.clearParamsInput();

		// Print out the command to cmd (and log file)
		scripterStream.writeLogFile(scripterStream.retStringCommand(),  false);
		
		// Process the command line
		if (!(interpreter.checkCommentLine(scripterStream.retStringCommand())))
		{
			// Increase the number of lines read if it is a command line
			++numOfLines;
			// Interpret the command's parameters
			interpreter.interpretCommandParams(scripterStream.retStringCommand());

			//First line has to be the LOG or MODE
			if (numOfLines == 1)
			{
				// If log mode is enable and the log file is not created yet, create the Log file
				if ((interpreter.retActualCommandParams().enLog) && (!logFileCreated))
				{
					scripterStream.initLogTxtFile(interpreter.retActualCommandParams().enLog, mainPathDir);
					numOfLines = 0;
					logFileCreated = true;
				}
				else
				{
					try
					{
						// Get the parameters of the MODE command and initiate
						execModeParameters();
					}
					catch (std::runtime_error& e)
					{
						return (exitProgramError(e.what()));
					}
				}
			}
			else
			{
				// Handling of the MODE command for the second time
				Family tempFamily = interpreter.retActualCommandParams().family;
				
				if (interpreter.retActualCommandParams().repeatMode)
				{
					try
					{
						getRepeatModeParameters(tempFamily);
					}
					catch (std::runtime_error& e)
					{
						return (exitProgramError(e.what()));
					}
				}  
				else
				{
					timer.resetTimeCount();
					
					// Match command with reference and read the parameters
					// Number of bytes received and transmitted on the communication are reset
					try
					{
						interpreter.matchCommand();
						targetInterface->readParameters(interpreter.retActualCommandParams());
						commInterface->resetCountBytes();
					}
					catch (std::runtime_error& e)
					{
						return (exitProgramError(e.what()));
					}

					// Handling of DELAY and VERBOSE command
					// These two commands do not need transmission to the target device
					if (interpreter.isCommandReqNoResponse())
					{
						try
						{
							targetInterface->callFunc(interpreter.retActualCommandParams(), 0);
							handleResponse();
						}
						catch (std::runtime_error& e)
						{
							return (exitProgramError(e.what()));
						}
					}
					
					// Handling of RX-commands that have input data file parameter
					else if (interpreter.isRxDataPassInputCommand())
					{
						// Check that the input data file is valid
						try
						{
							initReadingTxtFile();
						}
						catch (std::runtime_error& e)
						{
							return (exitProgramError(e.what()));
						}

						// Loop to read the input data file, store it in Data Block format,
						//    and send the data the target
						while (scripterStream.retReadStatus())
						{
							scripterStream.readTxtFile();

							if (scripterStream.retSendStatus())
							{
								targetInterface->setDataBlock(scripterStream.retDataBlockRead());
								try
								{
									executeBslCommand(0);	
								}
								catch (std::runtime_error& e)
								{
									return (exitProgramError(e.what()));
								}

								// Show and Log the sent-transmitted bytes when the Verbose mode is chosen
								showVerbose(targetInterface->retVerbose());//
							}
						}

						// Timer stopped and the speed of transmission is calculated
						//    Password transmission does not neet the calculation
						if (!interpreter.isRxPassword())
						{
							totalTime = timer.returnTimeCount();
							scripterStream.calcTransSpeed(totalTime, commInterface->retBytesWritten(), scripterStream.READ_MODE);
						}
					}
					// Handling of TX-commands that have input data file parameter
					else if (interpreter.isTxDataCommand())
					{
						// Number of bytes that will be applied for each single process
						uint32_t numReadBytes = 0; 
						// Number of bytes that will be received from the target device
						// It is be updated in the transmission process
						uint32_t numBytesLeft = targetInterface->retTxDataBlockSize();
						// Maximum number of bytes that is received for each single process
						uint16_t sizeBuffer = initWritingTxtFile(numBytesLeft);

						while (scripterStream.retBytesWritten() < targetInterface->retTxDataBlockSize())
						{
							//	Update the number of bytes that should be read, 
							//  and the number of bytes left to be processed on the next loop
							if (numBytesLeft > sizeBuffer)
							{
								numReadBytes = sizeBuffer;
							}
							else
							{
								numReadBytes = numBytesLeft;
							}
							numBytesLeft -= numReadBytes;  

							try
							{
								executeBslCommand(numReadBytes);
								scripterStream.writeTxtFile(ptrRxBuffer, numReadBytes);
							}
							catch (std::runtime_error& e)
							{
								return (exitProgramError(e.what()));
							}

							// Show and Log the sent-transmitted bytes when the Verbose mode is chosen
							showVerbose(targetInterface->retVerbose());
						}
						scripterStream.closeWriteTxtFile();
						totalTime = timer.returnTimeCount();
						scripterStream.calcTransSpeed(totalTime, targetInterface->retTxDataBlockSize(), scripterStream.WRITE_MODE);
					}
					// Handling of other commands
					else
					{
						// Set data block with the default password for RX_PASSWORD without
						// input data file parameter
						if (interpreter.isRxPassDefaultCommand())
						{
							switch (interpreter.retActualCommandParams().family)
							{
							case (Family::P4xx) :
								targetInterface->setDataBlock(defaultPass256);
								break;
							case (Family::F543x) :
								targetInterface->setDataBlock(defaultPass16);
								break;
							default:
								targetInterface->setDataBlock(defaultPass32);
								break;
							}
						}

						try
						{
							executeBslCommand(0);
							if (interpreter.retActualCommandParams().command == Command::CHANGE_BAUD_RATE)
							{
								interpreter.setupNewBaudRate(targetInterface->retNewBaudRate());
								commInterface->changeBaudRate(targetInterface->retNewBaudRate());
							}
							
						}
						catch (std::runtime_error& e)
						{
							return (exitProgramError(e.what()));
						}

						// Show and Log the sent-transmitted bytes when the Verbose mode is chosen
						showVerbose(targetInterface->retVerbose());
					}
					// Interpret the response and write to the log 
					handleResponse();
				}
			}
		}
	}
	return(exitProgram());
}

/*******************************************************************************
*Function:    execModeParameters
*Description: read the MODE command and its parameters
*             then build the respective target and communication
*Parameters:  none
*Returns:     none
*******************************************************************************/
void execModeParameters()
{
	interpreter.checkModeParameters();

	commInterface = factoryComm.createCommInterface(interpreter.retActualCommandParams().protocol);
	commInterface->init(interpreter.retActualCommandParams());

	targetInterface = factoryTarget.createTargetInterface(interpreter.retActualCommandParams().family);
	targetInterface->initVerbose();
}


/*******************************************************************************
*Function:    getRepeatModeParameters
*Description: read the MODE command and its parameters
*             after execModeParameters() already called
*Parameters:  Family prevFamSetting : previous family setting
*Returns:     none
*******************************************************************************/
void getRepeatModeParameters(Family prevFamSetting)
{
   	// Check parameters in the MODE command.
	interpreter.checkModeParameters();
	if (prevFamSetting != interpreter.retActualCommandParams().family)
	{
		throw std::runtime_error("Family of device does not match the previous setting...");
	}
	// Reinitiate the communication protocol. 
	commInterface->init(interpreter.retActualCommandParams());
}
/*******************************************************************************
*Function:    executeBslCommand
*Description: call the respective function to build buffer command
*             transmit the buffer command
*             receive the response bytes from BSL target device
*Parameters:  uint16_t numBytesCommand : expected number of response bytes
*Returns:     none
*******************************************************************************/
void executeBslCommand(uint16_t numBytesCommand)
{
	int32_t numOfBytesCast = (int32_t)numBytesCommand;
	if (!interpreter.isTxDataCommand())
	{
		targetInterface->callFunc(interpreter.retActualCommandParams(), 0);
		commInterface->transmitBuffer(targetInterface->retDataBuffer());
		ptrRxBuffer = commInterface->receiveBuffer(targetInterface->retNumResponseBytes());
	}
	else
	{
		targetInterface->callFunc(interpreter.retActualCommandParams(), numOfBytesCast);
		commInterface->transmitBuffer(targetInterface->retDataBuffer());
		ptrRxBuffer = commInterface->receiveBuffer(numOfBytesCast);
	}
}

/*******************************************************************************
*Function:    initReadingTxtFile
*Description: init the file to be read
*Parameters:  none
*Returns:     none
*******************************************************************************/
inline void initReadingTxtFile()
{
	scripterStream.initReadTxtFile(interpreter.retActualCommandParams(), mainPathDir);
	// Start the timer to count the operation of writing data to target process
	timer.startUpTimer();
}

/*******************************************************************************
*Function:    initWritingTxtFile
*Description: init the file to write data in
*Parameters:  uint32_t numBytesWrite: number of bytes that will be written
*             to the file
*Returns:     none
*******************************************************************************/
inline uint16_t initWritingTxtFile(uint32_t numBytesWrite)
{
	// Init streaming output file
	scripterStream.initWriteTxtFile(interpreter.retActualCommandParams(), mainPathDir);

	// Reset number bytes written
 	scripterStream.resetBytesWritten();
	scripterStream.writeLogFile(("Tx Data Block intended num of bytes : " + to_string(numBytesWrite)), true);

	// Maximum size to be received
	uint16_t sizeMaxRx = 65535;
	if ((interpreter.retActualCommandParams().protocol != Protocol::USB) ||
		(interpreter.retActualCommandParams().baudRate > 9600))
	{
		sizeMaxRx = 259;
	}

	// Reset the timer that will be used to count the elapsed transmission time.
	timer.resetTimeCount();
	timer.startUpTimer();

	return 	 sizeMaxRx;
}

/*******************************************************************************
*Function:	  changeBaudRate
*Description:
*Parameters:  none
*Returns:     none
*******************************************************************************/
void changeBaudRate()
{
	// CHANGE_BAUD_RATE requires a setup for the execCommunication
	interpreter.setupNewBaudRate(targetInterface->retNewBaudRate());
	commInterface->changeBaudRate(targetInterface->retNewBaudRate());
}

/*******************************************************************************
*Function:    showVerbose
*Description: call the function to show verbose mode if verbose is enabled
*Parameters:  bool enVerbose : enable verbose mode if true, otherwise disable
*Returns:     none
*******************************************************************************/
void showVerbose(bool enVerbose)
{
	if (enVerbose)
	{
		scripterStream.writeVerbose(targetInterface->retDataBuffer().data, targetInterface->retDataBuffer().size, scripterStream.RECT_BRACKET);
		scripterStream.writeVerbose(ptrRxBuffer, commInterface->retBytesPacketRead(), scripterStream.TRI_BRACKET);
	}
}

/*******************************************************************************
*Function:	  handleResponse
*Description: call the function to translate response bytes into
*             readable string response
*Parameters:  none
*Returns:     none
*******************************************************************************/
void handleResponse()
{
	if (!interpreter.isCommandReqNoResponse())
	{
		getResponseString = interpreter.retResponse(ptrRxBuffer, commInterface->retHeaderBuffer());
	}
	else
	{
		getResponseString = "";
	}
	scripterStream.writeLogFile(getResponseString, true);
}

/*******************************************************************************
*Function:	  exitProgram
*Description: routine to exit the program in successful condition
*             all streaming files would be closed
*Parameters:  none
*Returns:     none
*******************************************************************************/
int32_t exitProgram()
{
	scripterStream.closeReadTxtFile();
	scripterStream.closeWriteTxtFile();
	scripterStream.closeLogFile();
	commInterface->close();
	return (EXIT_PROGRAM);
}

/*******************************************************************************
*Function:	  exitProgramError
*Description: routine to exit the program in unsuccessful condition
*             the error message will be shown
*             all streaming files would be closed
*Parameters:  none
*Returns:     none
*******************************************************************************/
int32_t exitProgramError(std::string errorMessage)
{
	scripterStream.writeLogFile(errorMessage, true);
	scripterStream.closeReadTxtFile();
	scripterStream.closeWriteTxtFile();
	scripterStream.closeLogFile();
	commInterface->close();
	return(EXIT_PROGRAM_ERROR);
}