#pragma once

#include "stdafx.h"

class CommInterface 
{
public:

	/***********************************************************************************
	*Class:       CommInterface
	*Function:    resetCountBytes
	*Description: reset number of bytes that sent and received
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	void resetCountBytes();

	/***********************************************************************************
	*Class:       CommInterface
	*Function:    init
	*Description: do the initialization of the communication protocol based
	*             on the parameters of the given command
	*Parameters:  ActualCommandParams actualCommandParams: parameters from interpreter
	*Returns:     none
	***********************************************************************************/
	virtual void init(ActualCommandParams actualCommandParams) = 0;
	
	/***********************************************************************************
	*Class:       CommInterface
	*Function:    transmitBufferInit432
	*Description: do the invoke for MSP432 by sending 0xFF until the device answering
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	virtual void transmitBufferInit432() = 0;
	
	/***********************************************************************************
	*Class:       CommInterface
	*Function:    transmitBuffer
	*Description: send the BSL command 
	*Parameters:  DataBuffer dataBuffer : BSL command that is going to be sent
	*Returns:     none
	***********************************************************************************/
	virtual void transmitBuffer(DataBuffer dataBuffer) = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  receiveBuffer
	*Description: receive byte(s) response from target device
	*Parameters:  int32_t size : number of bytes that should be received by the
	*                            scripter
	*             (-1) means only one byte response without wrapped in BSL data packet
	*             (0) means no byte response
	*			  (n>0) means n bytes wrapped in BSL data packet 
	*Returns:     uint8_t* : pointer to the receive buffer 
	***********************************************************************************/
	virtual uint8_t* receiveBuffer(int32_t size) = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  close()
	*Description: close port communication
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	virtual void close() = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  changeBaudRate
	*Description: change baud rate value
	*Parameters:  uint32_t baudRate : new baud rate value
	*Returns:     none
	***********************************************************************************/
	virtual void changeBaudRate(uint32_t baudRate) = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  retBytesWritten()
	*Description: return number of bytes including the command byte
	*Parameters:  none
	*Returns:     uint32_t : number of bytes written to the device
	***********************************************************************************/
	virtual uint32_t retBytesWritten() = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  retBytesRead()
	*Description: return number of bytes (pure data without any command byte, check sum,
	*             or header bytes)
	*Parameters:  none
	*Returns:     uint32_t : number of bytes read from the device
	***********************************************************************************/
	virtual uint32_t retBytesRead() = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  retBytesPacketRead()
	*Description: return number of bytes included the command byte
	*Parameters:  none
	*Returns:     uint32_t : number of bytes read from the device
	***********************************************************************************/
	virtual uint32_t retBytesPacketRead() = 0;

	/***********************************************************************************
	*Class:       CommInterface
	*Function:	  retHeaderBuffer()
	*Description: return pointer of header buffer
	*Parameters:  none
	*Returns:     uint8_t* : pointer to the header buffer 
	***********************************************************************************/
	virtual uint8_t* retHeaderBuffer() = 0; 

protected:

	uint32_t bytesRead;  // number of bytes read (only data without any 
	                     //    command byte, check sum, or header bytes)	 
	uint32_t bytesWrite; // number of bytes written to the device including 
	                     //    the command byte
	uint32_t cnt;        // number of bytes read from the device including 
	                     //    the command byte

private:

};