#pragma once

#include "stdafx.h"

const uint32_t UART_RX_BUFFER_SIZE = 65790;
const uint32_t UART_TX_BUFFER_SIZE = 262;
const uint32_t UART_HEADER_BUFFER_SIZE = 4;

const uint32_t USB_RX_BUFFER_SIZE = 68760;
const uint32_t USB_TX_BUFFER_SIZE = 64;
const uint32_t USB_HEADER_BUFFER_SIZE = 2;

const uint32_t BUFFER_SIZE = 280;
const uint32_t PARAMS_SIZE = 6;

// DataBuffer is a struct that been used when sending/receiving data 
//    from the transmission.

struct DataBuffer{
	uint8_t  data[BUFFER_SIZE];	 // array to store the data
	uint32_t size;				 // size of the array that been used
};

// DataBlock is a struct that been used when reading/writing data 
//    to the file.

struct DataBlock{
	uint8_t  data[BUFFER_SIZE]; // array to store the data
	uint32_t numOfBytes;        // size data to be read/written
	uint32_t startAddress;		// start address where the data located in the memory
};

// CommandParams is a struct that used to declare the command
//    and its parameters.

struct CommandParams
{
	std::string commandText; // string of the command
	uint8_t numOfParamsMand; //	number of mandatory parameters
	uint8_t numOfParamsOpt;  // number of optional parameters
};

// ActualCommandParams is a struct that used to store the parameters
//    that been read from the scripter.

struct ActualCommandParams
{
	Family family;                        // family of the target device
	Protocol protocol;                    // communication protocol that is used
	uint32_t baudRate;                    // baudrate value for UART, I2C, and SPI protocol
	std::string commPort;                 // commport number that is used 
	uint8_t  repeatMode;                  // if MODE set is applied for the second time
	Command  command;                     // command that will be executed
	std::string paramsInput[PARAMS_SIZE]; // array to store parameters
	uint8_t  numOfParams;                 // number of available parameters
	uint16_t maxBufferSize;	              // maximum buffer size for sending data to the target device
	bool cryptoMode;                      // acknowledge if the CryptoBSL is chosen
	bool enLog;                           // enable the logging file of scripter process
	bool enParity;						  // enable the praity bit in UART transmission for certain devices
}; 
