#pragma once

#include "stdafx.h"

const uint32_t NUM_FAMILIES_AVAILABLE = 11;
const uint32_t NUM_PROTOCOLS_AVAILABLE = 4;
const uint32_t NUM_COMMANDS_AVAILABLE = 27;

const uint32_t NUM_BR_UART_AVAILABLE = 8;
const uint32_t NUM_BR_I2C_AVAILABLE = 2;
const uint32_t NUM_BR_SPI_AVAILABLE = 4;

// Program status

const int32_t EXIT_PROGRAM = 0;
const int32_t EXIT_PROGRAM_ERROR = -1;

// Note: Interpreter will convert the user's text to lower case format
//       therefore all definitions here written in lower case

// Family definition in strings	to be compared

const std::string familyDefs[NUM_FAMILIES_AVAILABLE] ={ 
	"543x_family", "msp430f543x",	//[0]  [1]
	"5xx", "msp430f5xx",			//[2]  [3]
	"6xx", "msp430f6xx",			//[4]  [5]
	"frxx", "msp430frxx",			//[6]  [7]
	"p4xx", "msp432p4xx",			//[8]  [9]
	"rom",							//[10] [11]
};

// Protocol definition in strings to be compared

const std::string protocolDefs[NUM_PROTOCOLS_AVAILABLE] ={ 
	"uart",
	"i2c",
	"spi",
	"usb"
};

// Commands definition in strings to be compared
// it is built with the	CommandParams struct defined in DataType.h

const CommandParams commandDefs[NUM_COMMANDS_AVAILABLE]={
	//command, number of mandatory parameters, number of optional parameters
	{ "change_baud_rate",1,0},//0
	{ "crc_check_32",2,1},//1
	{ "crc_check",2,1},//2
	{ "delay",1,0},//3
	{ "erase_check",2,0},//4
	{ "erase_main_or_info",1,0},//5
	{ "erase_segment_32",1,0},//6
	{ "erase_segment",1,0},//7
	{ "set_pc_32",1,0},//8
	{ "set_pc",1,0},//9
	{ "mass_erase",0,0},//10
	{ "reboot_reset",0,0},//11
	{ "rx_data_block_fast",1,0},//12
	{ "rx_data_block_32",1,0},//13
	{ "rx_data_block",1,0},//14
	{ "rx_password_32",0,1},//15
	{ "rx_password",0,1},//16
	{ "set_memory_offset",1,0},//17
	{ "toggle_info",0,0},//18
	{ "tx_data_block_32",3,0},//19
	{ "tx_data_block",3,0},//20
	{ "tx_bsl_version_32",0,0},//21
	{ "tx_bsl_version",0,0},//22
	{ "tx_buffer_size",0,0},//23
	{ "verbose",0,0},//24
	{ "rx_secure_data_block",1,0},//25
	{ "log",1,0 },//26
};

// Default 32 bytes password

const DataBlock defaultPass32 = {
		{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
		 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
		 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		 },
		32,
		0xFFE0
};

// Default 16 bytes password

const DataBlock defaultPass16 = {
		{0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
		16,
		0xFFF0
};

// Default 256 bytes password

const DataBlock defaultPass256 = {
		{ 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,},
		256,
		0x0000
};

// Values of valid baud rate for UART

const uint32_t baudRateUart[NUM_BR_UART_AVAILABLE] = {
	1200,
	2400,
	4800,
	9600,
	19200,
	38400,
	57600,
	115200
};

// Values of valid baud rate for I2C

const uint32_t baudRateI2c[NUM_BR_I2C_AVAILABLE] = {
	100000,
	400000
};

// Values of valid baud rate for SPI

const uint32_t baudRateSpi[NUM_BR_SPI_AVAILABLE] = {
	125000,
	250000,
	500000,
	1000000
};
