/*
BSL-Scripter-3.1.0.0

==RomTarget.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		08/07/2015		Fatmawati Santosa
3.1.0.0		10/26/2015		Fatmawati Santosa

*/

#include "stdafx.h"

/*******************************************************************************
*Class:       Rom
*Function:    readParameters
*Description: setup the parameters for each BSL command in Rom
*Parameters:  ActualCommandParams actualCommandParams:      get command
*Returns:     none
*******************************************************************************/
void RomTarget::readParameters(ActualCommandParams actualCommandParams)
{
	switch (actualCommandParams.command)
	{
	case (Command::DELAY) :
		delayMs = std::stoi(actualCommandParams.paramsInput[1]);
		break;

	case (Command::ERASE_SEGMENT) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::ERASE_CHECK) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::ERASE_MAIN_OR_INFO) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::RX_PASSWORD) :

		break;

	case(Command::SET_MEMORY_OFFSET) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::TX_DATA_BLOCK) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);

		totalBytesReceived = length;

		break;

	default:
		//no reading parameters
		break;
	}
}

/*******************************************************************************
*Class:       Rom
*Function:    changeBaudRate
*Description: Execution of CHANGE_BAUD_RATE in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::changeBaudRate()
{
	throw std::runtime_error("CHANGE_BAUD_RATE command is not available for ROM family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    crcCheck32
*Description: Execution of CRC_CHECK_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::crcCheck32()
{
	throw std::runtime_error("CRC_CHECK_32 command is not available for ROM family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    crcCheck
*Description: Execution of CRC_CHECK in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::crcCheck()
{
	throw std::runtime_error("CRC_CHECK command is not available for ROM family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    delay
*Description: Execution of DELAY in Rom family
*Parameters:  none
*Returns:     none
*******************************************************************************/
void RomTarget::delay()
{
	std::this_thread::sleep_for(std::chrono::milliseconds(delayMs));
}

/*******************************************************************************
*Class:       Rom
*Function:    eraseCheck
*Description: Execution of ERASE_CHECK in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::eraseCheck()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_ERASE_CHECK;
	commandBuffer.data[1] = 0x04;
	commandBuffer.data[2] = 0x04;
	commandBuffer.data[3] = (uint8_t)(address & 0xFF);
	commandBuffer.data[4] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[5] = (uint8_t)(length & 0xFF);
	commandBuffer.data[6] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 7; 

	return commandBuffer;
}

/*******************************************************************************
*Class:       Rom
*Function:    eraseMainOrInfo
*Description: Execution of ERASE_MAIN_OR_INFO in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::eraseMainOrInfo()
{
	throw std::runtime_error("ERASE_MAIN_OR_INFO command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    eraseSegment32
*Description: Execution of ERASE_SEGMENT_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::eraseSegment32()
{
	throw std::runtime_error("ERASE_SEGMENT_32 command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    eraseSegment
*Description: Execution of ERASE_SEGMENT in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::eraseSegment()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_ERASE_SEGMENT;
	commandBuffer.data[1] = 0x04;
	commandBuffer.data[2] = 0x04;
	commandBuffer.data[3] = (uint8_t)(address & 0x00FF);
	commandBuffer.data[4] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[5] = 0x02;
	commandBuffer.data[6] = 0xA5;

	commandBuffer.size = 7;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       Rom
*Function:    setPc32
*Description: Execution of SET_PC_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::setPc32()
{
	throw std::runtime_error("SET_PC_32 command is not available for ROM Family..");
}

/*******************************************************************************
*Class:       Rom
*Function:    setPc
*Description: Execution of SET_PC in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::setPc()
{
	throw std::runtime_error("SET_PC command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    massErase
*Description: Execution of MASS_ERASE in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::massErase()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_MASS_ERASE;
	commandBuffer.data[1] = 0x04;
	commandBuffer.data[2] = 0x04;
	commandBuffer.data[3] = 0x00;
	commandBuffer.data[4] = 0xFF;
	commandBuffer.data[5] = 0x06;
	commandBuffer.data[6] = 0xA5;

	commandBuffer.size = 7;

	numResponseBytes = 0;

	return commandBuffer;
}

/*******************************************************************************
*Class:       Rom
*Function:    rebootReset
*Description: Execution of REBOOT_RESET in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::rebootReset()
{
	throw std::runtime_error("REBOOT_RESET command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    rxDataBlockFast
*Description: Execution of RX_DATA_BLOCK_FAST in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::rxDataBlockFast()
{
	throw std::runtime_error("RX_DATA_BLOCK command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    rxDataBlock32
*Description: Execution of RX_DATA_BLOCK_FAST_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::rxDataBlock32()
{
	throw std::runtime_error("RX_DATA_BLOCK_32 command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    rxDataBlock
*Description: Execution of RX_DATA_BLOCK in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::rxDataBlock()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_RX_DATA_BLOCK;
	commandBuffer.data[1] = (uint8_t)((dataBlock.numOfBytes + 4) & 0xFF);
	commandBuffer.data[2] = (uint8_t)((dataBlock.numOfBytes + 4) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((dataBlock.startAddress) & 0xFF);
	commandBuffer.data[4] = (uint8_t)((dataBlock.startAddress >> 8) & 0xFF);
	commandBuffer.data[5] = (uint8_t)((dataBlock.numOfBytes) & 0xFF);
	commandBuffer.data[6] = 0x00;

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 7] = dataBlock.data[i];
	}

	commandBuffer.size = dataBlock.numOfBytes + 7;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       Rom
*Function:    rxPassword32
*Description: Execution of RX_PASSWORD_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::rxPassword32()
{
	throw std::runtime_error("RX_PASSWORD_32 command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    rxPassword
*Description: Execution of RX_PASSWORD in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::rxPassword()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_RX_PASSWORD;
	commandBuffer.data[1] = 0x24;
	commandBuffer.data[2] = 0x24;
	commandBuffer.data[3] = ROM_DUMMY_DATA;
	commandBuffer.data[4] = ROM_DUMMY_DATA;
	commandBuffer.data[5] = ROM_DUMMY_DATA;
	commandBuffer.data[6] = ROM_DUMMY_DATA;


	commandBuffer.data[0] = RX_PASSWORD;

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 7] = dataBlock.data[i];
	}

	commandBuffer.size = 7 + dataBlock.numOfBytes;

	numResponseBytes = 1;

	return commandBuffer;
}

/*******************************************************************************
*Class:       Rom
*Function:    setMemoryOffset
*Description: Execution of SET_MEM_OFFSET in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::setMemoryOffset()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_SET_MEM_OFFSET;
	commandBuffer.data[1] = 0x04;
	commandBuffer.data[2] = 0x04;
	commandBuffer.data[3] = ROM_DUMMY_DATA;
	commandBuffer.data[4] = ROM_DUMMY_DATA;
	commandBuffer.data[5] = (uint8_t)(address & 0x00FF);
	commandBuffer.data[6] = (uint8_t)((address >> 8) & 0x00FF);

	commandBuffer.size = 7;

	return commandBuffer;
}

/*******************************************************************************
*Class:       Rom
*Function:    toggleInfo
*Description: Execution of TOGGLE_INFO in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::toggleInfo()
{
	throw std::runtime_error("TOGGLE_INFO command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    txDataBlock32
*Description: Execution of TX_DATA_BLOCK_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::txDataBlock32(uint16_t size)
{
	throw std::runtime_error("TX_DATA_BLOCK_32 command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    txDataBlock
*Description: Execution of TX_DATA_BLOCK in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::txDataBlock(uint16_t size)
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ROM_TX_DATA_BLOCK;
	commandBuffer.data[1] = 0x04;
	commandBuffer.data[2] = 0x04;
	commandBuffer.data[3] = (uint8_t)(address & 0xFF);
	commandBuffer.data[4] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[5] = (uint8_t)size;
	commandBuffer.data[6] = 0;

	commandBuffer.size = 7;

	numResponseBytes = totalBytesReceived;

	return commandBuffer;
}

/*******************************************************************************
*Class:
*Function:
*Description:
*Parameters:
*Returns:     D
*******************************************************************************/
DataBuffer RomTarget::rxSecureDataBlock()
{
	throw std::runtime_error("BRX_SECURE_DATA_BLOCK command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    bslVersion32
*Description: Execution of TX_BSL_VERSION_32 in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::bslVersion32()
{
	throw std::runtime_error("BSL_VERSION_32 command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    bslVersion
*Description: Execution of TX_BSL_VERSION in Rom family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer RomTarget::bslVersion()
{
	throw std::runtime_error("BSL_VERSION command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    txBufferSize
*Description: Execution of TX_BUFFER_SIZE in Rom family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer RomTarget::txBufferSize()
{
	throw std::runtime_error("TX_BUFFER_SIZE command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    initVerbose
*Description: set the verbose mode to off as initialization
*Parameters:  none
*Returns:     none
*******************************************************************************/
void RomTarget::initVerbose()
{
	verboseToggle = false;
}

/*******************************************************************************
*Class:       Rom
*Function:    verbose
*Description: toggle verbose mode
*Parameters:  none
*Returns:     none
*******************************************************************************/
void RomTarget::verbose()
{
	if (verboseToggle == false)
	{
		verboseToggle = true;
	}
	else
	{
		verboseToggle = false;
	}

}

/*******************************************************************************
*Class:       Rom
*Function:    retVerbose
*Description: return verbose mode
*Parameters:  none
*Returns:     bool true : verbose on
*             bool false: verbose off
*******************************************************************************/
bool RomTarget::retVerbose()
{
	return verboseToggle;
}

/*******************************************************************************
*Class:       Rom
*Function:    retNewBaudRate
*Description: return baud rate value
*Parameters:  none
*Returns:     exception
*******************************************************************************/
uint32_t RomTarget::retNewBaudRate()
{
	throw std::runtime_error("CHANGE_BAUD_RATE command is not available for ROM Family...");
}

/*******************************************************************************
*Class:       Rom
*Function:    retNumResponseBytes
*Description: return number of response byte of respective command
*Parameters:  none
*Returns:     uint32_t numResponseBytes :umber of response byte
*******************************************************************************/
int32_t RomTarget::retNumResponseBytes()
{
	return numResponseBytes;
}

/*******************************************************************************
*Class:       Rom
*Function:    setDataBlock
*Description: copy DataBlock from streaming input to be applied in BSL command
*Parameters:  DataBlock dataBlockStream: input DataBlock from streaming input
*Returns:     none
*******************************************************************************/
void RomTarget::setDataBlock(DataBlock dataBlockStream)
{
	dataBlock = dataBlockStream;
}

/*******************************************************************************
*Class:       Rom
*Function:
*Description:
*Parameters:
*Returns:
*******************************************************************************/
uint32_t RomTarget::retTxDataBlockSize()
{
	return totalBytesReceived;
}