/*
BSL-Scripter-3.1.0.0

==ScripterStream.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		08/07/2015		Fatmawati Santosa
3.1.0.0		10/26/2015		Fatmawati Santosa

*/

#include "stdafx.h"

//Constructor
ScripterStream::ScripterStream()
{
	for (uint16_t i = 0; i < BUFFER_SIZE; i++)
	{
		dataBlockRead.data[i] = 0;
	}
	dataBlockRead.startAddress = 0;
	dataBlockRead.numOfBytes = 0;

	fileScriptName = "";
	line = "";
}

ScripterStream::~ScripterStream()
{

}
 
std::string ScripterStream::startScripter(char * ch)
{
	if (ch == nullptr)
	{
		ScripterStream::writeLogFile("Enter the name of file script : ", false);
		std::cin >> fileName;											
	}
	else
	{
		fileName = (reinterpret_cast<char*>(ch));
	}

	boost::filesystem::path fullPath(boost::filesystem::current_path());
	
	std::string pathDir = fullPath.string();
	pathDir += "/addition";

	std::string path = getThePathFile(fileName, pathDir);
	std::string showFileName = path;
	std::replace(showFileName.begin(), showFileName.end(), '\\', '/');

	date_time = boost::posix_time::second_clock::local_time();
	std::stringstream streamLocalTime;
	streamLocalTime << date_time;

	showHeaderFile = "---------------------------------------------------------\nBSL Scripter 3.1.0.0\nPC software for BSL programming\n" +
					 streamLocalTime.str() + "\n" +
					 "---------------------------------------------------------\n" +
					 ("Input file script is : " + showFileName) + "\n";

	ScripterStream::writeLogFile("---------------------------------------------------------",false);
	ScripterStream::writeLogFile("BSL Scripter 3.1.0.0", false);
	ScripterStream::writeLogFile("PC software for BSL programming", false);
	ScripterStream::writeLogFile(streamLocalTime.str(), false);
	ScripterStream::writeLogFile("---------------------------------------------------------",  false);
	ScripterStream::writeLogFile(("Input file script is : " + showFileName), false);
 
	fileScriptStream.open(path, ios::in);
	if (!fileScriptStream.good())
	{
		throw std::runtime_error("File Scripter is not available...");
	}
	return path;
}
 
bool ScripterStream::getLine()
{
	if (fileScriptStream.eof())
	{
		return false;
	}
	else
	{
		std::getline(fileScriptStream, line);
		return true;
	}
}


std::string ScripterStream::retStringCommand()
{
	return line;
}


void ScripterStream::initReadTxtFile(ActualCommandParams actualCommandParams, std::string mainPathDir)
{
	if ((actualCommandParams.family == Family::P4xx) && (actualCommandParams.command == Command::RX_PASSWORD_32)
		&& (actualCommandParams.paramsInput[1] != ""))
	{
		numBytesToRead = 256;
	}
	else if (actualCommandParams.command == Command::RX_SECURE_DATA_BLOCK)
	{
		numBytesToRead = 256;
	}
	else
	{
		numBytesToRead = actualCommandParams.maxBufferSize;
	}

	fileName = getThePathFile(actualCommandParams.paramsInput[1], mainPathDir);
	std::string showFileName = fileName;
	std::replace(showFileName.begin(), showFileName.end(), '\\', '/');
	ScripterStream::writeLogFile(("Read Txt File  : " + showFileName), true);
	bytesRead = 0;
	onGoingReadStatus();
	clearSendStatus();
	changeAddress = NO_CHANGE_ADDRESS;

	fileReadStream.open(fileName, ios::in);
	if (!fileReadStream.good())
	{
		throw std::runtime_error("File is not available/accessed by other program...");
	}
}

void ScripterStream::readTxtFile()
{
	std::string   tempLine;
	std::string   tempToken;

	if (bytesRead < numBytesToRead)
	{
		if (!fileReadStream.eof())
		{
			std::getline(fileReadStream, tempLine);

			boost::char_separator<char> sep("\t\r ");
			boost::tokenizer<boost::char_separator<char>> tokens(tempLine, sep);

			for (const string& t : tokens)
			{
				tempToken = t;
				uint8_t pos = tempToken.find_first_of("@");
				if (pos == 0)
				{
					clearSendStatus();
					tempToken.erase(0, 1);
					saveLastReadAddress = std::stoul(tempToken, nullptr, 16);
					changeAddress = CHANGE_ADDRESS;

					if (bytesRead != 0)
					{
						setSendStatus();
						dataBlockRead.numOfBytes = bytesRead;
						bytesRead = 0;
						return;
					}
				}
				//continue read the data on the same address
				else
				{
					if ((tempToken == "q") || (tempToken == "Q"))
					{
						dataBlockRead.numOfBytes = bytesRead;
						setSendStatus();
						finishReadStatus();
						fileReadStream.close();
						return;
					}
					else
					{
						onGoingReadStatus();
						clearSendStatus();

						if (changeAddress == CHANGE_ADDRESS)
						{
							dataBlockRead.startAddress = saveLastReadAddress;
							changeAddress = NO_CHANGE_ADDRESS;
						}

						dataBlockRead.data[bytesRead] = (uint8_t)(std::stoul(tempToken, nullptr, 16));
						bytesRead++;

						if (bytesRead == numBytesToRead)
						{
							dataBlockRead.numOfBytes = bytesRead;
							saveLastReadAddress = dataBlockRead.numOfBytes + dataBlockRead.startAddress;
							changeAddress = CHANGE_ADDRESS;
							bytesRead = 0;
							setSendStatus();
							return;
						}
					}
				}
			}
		}
	}
}

DataBlock ScripterStream::retDataBlockRead()
{
	return dataBlockRead;
}

uint8_t ScripterStream::retSendStatus()
{
	return sendStatus;
}


uint8_t ScripterStream::retReadStatus()
{
	return readStatus;
}


void ScripterStream::closeReadTxtFile()
{
	fileReadStream.close();
}


void ScripterStream::initWriteTxtFile(ActualCommandParams actualCommandParams, std::string mainPathDir)
{
	fileName = getThePathFile(actualCommandParams.paramsInput[3], mainPathDir);
	std::string showFileName = fileName;
	std::replace(showFileName.begin(), showFileName.end(), '\\', '/');
	ScripterStream::writeLogFile(("Write Txt File : " + showFileName), true);
	fileWriteStream.open(fileName, ios::out);
	actualCommandParams.paramsInput[1].erase(0, 2);
	fileWriteStream << "@" << actualCommandParams.paramsInput[1] << "\n";

	if (actualCommandParams.protocol == Protocol::USB)
	{
		maxBufSize = 62;
	}
	else
	{
		maxBufSize = 260;
	}

	if (!fileWriteStream.good())
	{
		throw std::runtime_error("File to write the file could not be created...");
	}
}		  
  
void ScripterStream::writeTxtFile(uint8_t *buf, uint32_t size)
{
	k = 0;
	float numOfHeaderByte = (float)size / (maxBufSize - 1);
	uint32_t totalByte = (uint32_t)(ceil(numOfHeaderByte)) + size;

	for (uint32_t i = 0; i < totalByte; i++, buf++)
	{
		if (k != 0)
		{
			std::string tempString = intToHex(*buf);
			boost::to_upper(tempString);
			fileWriteStream << tempString;
			bytesWritten++;

			if (bytesWritten % 16 == 0)
			{
				fileWriteStream << "\n";
			}
			else
			{
				fileWriteStream << " ";
			}
		}
		k++;
		if (k == maxBufSize)
		{
			k = 0;
		}
	}
}

void ScripterStream::resetBytesWritten(void)
{
	bytesWritten = 0;
	k = 0;
}

uint32_t ScripterStream::retBytesWritten(void)
{
	return bytesWritten;
}

void ScripterStream::closeWriteTxtFile()
{
	if (bytesWritten % 16 != 0)
	{
		fileWriteStream << "\n";
	}
	fileWriteStream << "q";

	fileWriteStream.close();
}

void ScripterStream::initLogTxtFile(bool enLogInterpreter, std::string path)
{
	enLog = enLogInterpreter;
	if (enLog)
	{
		std::stringstream streamDate;
		streamDate << date_time.date();
		std::string date = streamDate.str();

		std::stringstream streamTime;
		streamTime << date_time.time_of_day();
		std::string time = streamTime.str();

		std::string logPath = path;
		boost::replace_all(date, "-", "");
		boost::replace_all(time, ":", "");
		boost::replace_all(logPath, ".txt", "");
		logPath += "_" + date + "_" + time + ".log";

		fileLogStream.open(logPath, ios::out);
		if (!fileLogStream.good())
		{
			throw std::runtime_error("Failed to create the log file...");
		}
		fileLogStream << showHeaderFile;
	}

}

void ScripterStream::writeLogFile(std::string line, bool tab)
{
	std::string showLine;
	if (tab)
	{
		showLine = "\t" + line;
	}
	else
	{
		showLine = line;
	}

	if (enLog && fileLogStream.good())
	{
		fileLogStream << showLine << std::endl;
	}

	std::cout << showLine << std::endl;
}

void ScripterStream::writeVerbose(uint8_t *buf, uint32_t bufSize, uint8_t bracket)
{
	std::string bracket_first;
	std::string bracket_end;
	if (bracket == RECT_BRACKET)
	{
		bracket_first = "[";
		bracket_end = "] ";
	}
	else
	{
		bracket_first = "<";
		bracket_end = "> ";
	}

	if (bufSize > 0)
	{
		if (enLog && fileLogStream.good()) fileLogStream << "\t";
		std::cout << "\t";

		for (uint32_t i = 0; i < bufSize; i++, buf++)
		{
			std::string tempStr = intToHex(*buf);
			std::string completeStr = bracket_first + tempStr + bracket_end;

			if (enLog && fileLogStream.good()) fileLogStream << completeStr;
			std::cout << completeStr;

			if ((i + 1) % 16 == 0)
			{
				if (enLog && fileLogStream.good()) fileLogStream << "\n";
				std::cout << "\n";

				if (enLog && fileLogStream.good()) fileLogStream << "\t";
				std::cout << "\t";
			}
		}
	}
	if (enLog && fileLogStream.good()) fileLogStream << "\n";
	std::cout << "\n";
}

void ScripterStream::closeLogFile()
{
	if (enLog)
	{
		fileLogStream.close();
	}
}

void ScripterStream::calcTransSpeed(double time, uint32_t numOfbytes, uint8_t mode)
{
	double cntSpeed = (((double)numOfbytes) / 1024.0) / time;
	std::stringstream streamCntSpeed;
	streamCntSpeed << std::setprecision(4) << cntSpeed;
	std::stringstream streamTotalTime;
	streamTotalTime << std::setprecision(4) << time;

	std::string textRead1 = "Time elapsed of writing " + to_string(numOfbytes) + " bytes : " + streamTotalTime.str() + " seconds";
	std::string textWrite1 = "Time elapsed of reading " + to_string(numOfbytes) + " bytes : " + streamTotalTime.str() + " seconds";
	std::string textRead2 = "Speed of writing data :" + streamCntSpeed.str() + "(kB/s)";
	std::string textWrite2 = "Speed of reading data : " + streamCntSpeed.str() + "(kB/s)";

	//write to logfile and show in the screen
	if (mode == READ_MODE)
	{
		writeLogFile(textRead1, true);
		writeLogFile(textRead2, true);
	}
	else
	{
		writeLogFile(textWrite1, true);
		writeLogFile(textWrite2, true);
	}
}


std::string ScripterStream::getThePathFile(std::string stInput, std::string relativePath)
{
	std::string resultPath = "";

#if defined(_WIN32) || defined(_WIN64)
	const char* sepSymbol = "\\";
	const char* sepSymbol_ = "/";
	uint8_t pos_doubleDots = stInput.find_first_of(":");
	bool goAbsolute = false;
	if (pos_doubleDots == 1)
	{
		goAbsolute = true;
	}
#else
	const char* sepSymbol = "/";
	uint8_t pos_doubleDots = stInput.find_first_of("/");
	bool goAbsolute = false;
	if (pos_doubleDots == 0)
	{
		goAbsolute = true;
	}
#endif

	uint8_t pos_upFolder = stInput.find_first_of("..");

	if (goAbsolute)//if (pos_doubleDots == 1)  // ABSOLUTE PATH
	{
		resultPath = stInput;
	}
	else // UP_FOLDER
	{
		boost::char_separator<char> sep("\\,/");
		boost::tokenizer<boost::char_separator<char>> tokensRelative(relativePath, sep);

		uint8_t cntDeepRelativePath = 0;
		std::string tempBufString[10];

		for (const string& t : tokensRelative)
		{
			tempBufString[cntDeepRelativePath] = t;
			cntDeepRelativePath++;
		}

		if (pos_upFolder == 0)   //UP_FOLDER
		{
			boost::tokenizer<boost::char_separator<char>> tokens(stInput, sep);

			uint8_t cntUpFolder = 0;
			std::string strTemp = "";
			for (const string& t : tokens)
			{
				if (t == "..")
				{
					cntUpFolder++;
				}
				else
				{
					strTemp = t;
					break;
				}
			}

			for (uint8_t i = 0; i < (cntDeepRelativePath - cntUpFolder - 1); i++)
			{
				resultPath += tempBufString[i] + sepSymbol;
			}
			resultPath += strTemp;
		}
		else // DOWN FOLDER
		{
			for (uint8_t i = 0; i < (cntDeepRelativePath - 1); i++)
			{
				resultPath += tempBufString[i] + sepSymbol;
			}
			resultPath += stInput;
		}
	}
#if defined(_WIN32) || defined(_WIN64)
	return resultPath;
#else
	std::string temp = "/" + resultPath;
	return temp;
#endif
}
 
void ScripterStream::clearSendStatus()
{
	sendStatus = KEEP_DATA_BLOCK;
}
 
void ScripterStream::setSendStatus()
{
	sendStatus = SEND_DATA_BLOCK;
}

void ScripterStream::onGoingReadStatus()
{
	readStatus = READ_ON_GOING;
}

void ScripterStream::finishReadStatus()
{
	readStatus = READ_DONE;
}
 
std::string ScripterStream::intToHex(uint8_t input)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::setw(sizeof(input) * 2)
		<< std::hex
		<< (uint32_t)input;
	return stream.str();
}

std::string ScripterStream::intToHexAddr(uint32_t address)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::hex
		<< address;
	return stream.str();
}
