#pragma once

#include "stdafx.h"
#include "hidapi.h"
#include "DataType.h"

class UsbComm : public CommInterface
{
public:

	UsbComm();
	~UsbComm();

	/*******************************************************************************
	*Class:       UartComm
	*Function:	  init
	*Description: Setup the initialization of uart communication
	*Parameters:  ActualCommandParams actualCommandParams
	*Returns:	  none
	*******************************************************************************/
	void init(ActualCommandParams actualCommandParams) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:    transmitBufferInit432
	*Description: do the invoke for MSP432 by sending 0xFF until the device answering
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	void transmitBufferInit432() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:    transmitBuffer
	*Description: send the BSL command
	*Parameters:  DataBuffer dataBuffer : BSL command that is going to be sent
	*Returns:     none
	***********************************************************************************/
	void transmitBuffer(DataBuffer dataBuffer) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  receiveBuffer
	*Description: receive byte(s) response from target device
	*Parameters:  int32_t size : number of bytes that should be received by the
	*                            scripter
	*             (-1) means only one byte response without wrapped in BSL data packet
	*             (0) means no byte response
	*			  (n>0) means n bytes wrapped in BSL data packet
	*Returns:     uint8_t* : pointer to the receive buffer
	***********************************************************************************/
	uint8_t* receiveBuffer(int32_t size) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  close()
	*Description: close port communication
	*Parameters:  none
	*Returns:     none
	***********************************************************************************/
	void close() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  changeBaudRate
	*Description: change baud rate value
	*Parameters:  uint32_t baudRate : new baud rate value
	*Returns:     none
	***********************************************************************************/
	void changeBaudRate(uint32_t baudRate) override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retBytesWritten()
	*Description: return number of bytes including the command byte
	*Parameters:  none
	*Returns:     uint32_t : number of bytes written to the device
	***********************************************************************************/
	uint32_t retBytesWritten() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retBytesRead()
	*Description: return number of bytes (pure data without any command byte, check sum,
	*             or header bytes)
	*Parameters:  none
	*Returns:     uint32_t : number of bytes read from the device
	***********************************************************************************/
	uint32_t retBytesRead() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retBytesPacketRead()
	*Description: return number of bytes included the command byte
	*Parameters:  none
	*Returns:     uint32_t : number of bytes read from the device
	***********************************************************************************/
	uint32_t retBytesPacketRead() override;

	/***********************************************************************************
	*Class:       UartComm
	*Function:	  retHeaderBuffer()
	*Description: return pointer of header buffer
	*Parameters:  none
	*Returns:     uint8_t* : pointer to the header buffer
	***********************************************************************************/
	uint8_t* retHeaderBuffer() override;

private:

	const uint32_t MS_DELAY_RX = 500;
	// TI BSL Device ID
	const uint16_t VID = 0x2047; 
	const uint16_t PID = 0x0200; 
	
	hid_device *handle; 

	uint8_t headerBuffer[USB_HEADER_BUFFER_SIZE];
	uint8_t rxBuffer[USB_RX_BUFFER_SIZE];

	void cleanRxBuffer();
	void cleanHeaderBuffer();
};

